@@ -2,33 +2,38 @@
 
 Copyright (c) 1999-2004 Ng Pheng Siong. All rights reserved.
 
-Portions Copyright (c) 2004 Open Source Applications Foundation.
+Portions Copyright (c) 2004-2006 Open Source Applications Foundation.
 Author: Heikki Toivonen
 """
 
-RCS_id='$Id: EVP.py 285 2005-06-03 08:13:44Z heikki $'
-
-from M2Crypto import Err, util, BIO
+from M2Crypto import Err, util, BIO, RSA
 import m2
 
 class MessageDigest:
     """
     Message Digest
     """
+    m2_md_ctx_free = m2.md_ctx_free
+
     def __init__(self, algo):
-        md = getattr(m2, algo)
-        if not md:
+        md = getattr(m2, algo, None)
+        if md is None:
             raise ValueError, ('unknown algorithm', algo)
         self.md=md()
         self.ctx=m2.md_ctx_new()
         m2.digest_init(self.ctx, self.md)
-
+        
     def __del__(self):
-        if self.ctx:
-            m2.md_ctx_free(self.ctx)
+        if getattr(self, 'ctx', None):
+            self.m2_md_ctx_free(self.ctx)
 
     def update(self, data):
-        m2.digest_update(self.ctx, data)
+        """
+        Add data to be digested.
+        
+        @return: -1 for Python error, 1 for success, 0 for OpenSSL failure.
+        """
+        return m2.digest_update(self.ctx, data)
 
     def final(self):
         return m2.digest_final(self.ctx)
@@ -38,17 +43,20 @@ class MessageDigest:
 
 
 class HMAC:
+    
+    m2_hmac_ctx_free = m2.hmac_ctx_free
+
     def __init__(self, key, algo='sha1'):
-        md = getattr(m2, algo)
-        if not md:
+        md = getattr(m2, algo, None)
+        if md is None:
             raise ValueError, ('unknown algorithm', algo)
         self.md=md()
         self.ctx=m2.hmac_ctx_new()
         m2.hmac_init(self.ctx, key, self.md)
-
+        
     def __del__(self):
-        if self.ctx:
-            m2.hmac_ctx_free(self.ctx)
+        if getattr(self, 'ctx', None):
+            self.m2_hmac_ctx_free(self.ctx)
 
     def reset(self, key):
         m2.hmac_init(self.ctx, key, self.md)
@@ -62,30 +70,33 @@ class HMAC:
     digest=final
 
 def hmac(key, data, algo='sha1'):
-    md = getattr(m2, algo)
-    if not md:
+    md = getattr(m2, algo, None)
+    if md is None:
         raise ValueError, ('unknown algorithm', algo)
     return m2.hmac(key, data, md())
 
 
 class Cipher:
+
+    m2_cipher_ctx_free = m2.cipher_ctx_free
+
     def __init__(self, alg, key, iv, op, key_as_bytes=0, d='md5', salt='12345678', i=1):
-        cipher = getattr(m2, alg)
-        if not cipher:
+        cipher = getattr(m2, alg, None)
+        if cipher is None:
             raise ValueError, ('unknown cipher', alg)
         self.cipher=cipher()
         if key_as_bytes:
-            kmd = getattr(m2, d)
-            if not kmd:
+            kmd = getattr(m2, d, None)
+            if kmd is None:
                 raise ValueError, ('unknown message digest', d)
             key = m2.bytes_to_key(self.cipher, kmd(), key, salt, iv, i)
         self.ctx=m2.cipher_ctx_new()
         m2.cipher_init(self.ctx, self.cipher, key, iv, op)
         del key
-
+        
     def __del__(self):
-        if self.ctx:
-            m2.cipher_ctx_free(self.ctx)
+        if getattr(self, 'ctx', None):        
+            self.m2_cipher_ctx_free(self.ctx)
 
     def update(self, data):
         return m2.cipher_update(self.ctx, data)
@@ -98,6 +109,10 @@ class PKey:
     """
     Public Key
     """
+    
+    m2_pkey_free = m2.pkey_free
+    m2_md_ctx_free = m2.md_ctx_free
+
     def __init__(self, pkey=None, _pyfree=0, md='sha1'):
         if pkey is not None:
             self.pkey = pkey
@@ -106,18 +121,19 @@ class PKey:
             self.pkey = m2.pkey_new()
             self._pyfree = 1
         self._set_context(md)
-
+        
     def __del__(self):
-        if self._pyfree:
-            m2.pkey_free(self.pkey)
-        m2.md_ctx_free(self.ctx)
+        if getattr(self, '_pyfree', 0):
+            self.m2_pkey_free(self.pkey)
+        if getattr(self, 'ctx', None):
+            self.m2_md_ctx_free(self.ctx)
 
     def _ptr(self):
         return self.pkey
 
     def _set_context(self, md):
-        mda = getattr(m2, md)
-        if not mda:
+        mda = getattr(m2, md, None)
+        if mda is None:
             raise ValueError, ('unknown message digest', md)
         self.md = mda()
         self.ctx = m2.md_ctx_new()
@@ -171,8 +187,9 @@ class PKey:
 
         @type data: string
         @param data: Data to be verified.
+        @return: -1 on Python error, 1 for success, 0 for OpenSSL error
         """
-        m2.verify_update(self.ctx, data)
+        return m2.verify_update(self.ctx, data)
 
     def verify_final(self):
         """
@@ -206,6 +223,18 @@ class PKey:
             ret = m2.pkey_set1_rsa(self.pkey, rsa.rsa)
         return ret
 
+    def get_rsa(self):
+        """
+        Return the underlying RSA key if that is what the EVP
+        instance is holding.
+        """
+        rsa_ptr = m2.pkey_get1_rsa(self.pkey)
+        if rsa_ptr is None:
+            raise ValueError("PKey instance is not holding a RSA key")
+        
+        rsa = RSA.RSA(rsa_ptr, 1)
+        return rsa
+
     def save_key(self, file, cipher='aes_128_cbc', callback=util.passphrase_callback):
         """
         Save the key pair to a file in PEM format.
@@ -269,6 +298,26 @@ class PKey:
         self.save_key_bio(bio, cipher, callback)
         return bio.read_all()
 
+    def as_der(self):
+        """
+        Return key in DER format in a string
+        """
+        buf = m2.pkey_as_der(self.pkey)
+        bio = BIO.MemoryBuffer(buf)
+        return bio.read_all()
+   
+    def size(self):
+        """
+        Return the size of the key in bytes.
+        """
+        return m2.pkey_size(self.pkey)
+        
+    def get_modulus(self):
+        """
+        Return the modulus in hex format.
+        """
+        return m2.pkey_get_modulus(self.pkey)
+   
 
 def load_key(file, callback=util.passphrase_callback):
     """
