@@ -8,7 +8,7 @@ Copyright (C) 2004-2006 OSAF. All Rights Reserved.
 
 """
 
-version_info = (0, 16)
+version_info = (0, 17)
 version = '.'.join([str(v) for v in version_info])
 
 import __m2crypto
@@ -33,6 +33,12 @@ import PGP
 import m2urllib
 # Backwards compatibility.
 urllib2 = m2urllib
+
+import sys
+if sys.version_info >= (2,4):
+    import m2urllib2
+del sys
+
 import ftpslib
 import httpslib
 import m2xmlrpclib
