@@ -4,15 +4,12 @@
 
 Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""
 
-RCS_id='$Id: test_dh.py 299 2005-06-09 17:32:28Z heikki $'
-
 import unittest
 from M2Crypto import DH, BIO, Rand, m2
-import sys
 
 class DHTestCase(unittest.TestCase):
 
-    params = 'dhparam.pem'
+    params = 'tests/dhparam.pem'
 
     def genparam_callback(self, *args):
         pass 
@@ -36,20 +33,15 @@ class DHTestCase(unittest.TestCase):
         bio = BIO.MemoryBuffer()
         a.print_params(bio)
         params = bio.read()
-        if sys.version[:3] in ('2.0', '2.1'):
-            assert params.find('(128 bit)')
-            assert params.find('generator: 2 (0x2)')
-        elif sys.version[:3] == '1.5': 
-            import string
-            assert string.find(params, '(128 bit)')
-            assert string.find(params, 'generator: 2 (0x2)')
+        assert params.find('(128 bit)')
+        assert params.find('generator: 2 (0x2)')
 
     def check_load_params(self):
-        a = DH.load_params('dhparams.pem')
+        a = DH.load_params('tests/dhparams.pem')
         assert a.check_params() == 0
 
     def check_compute_key(self):
-        a = DH.load_params('dhparams.pem')
+        a = DH.load_params('tests/dhparams.pem')
         b = DH.set_params(a.p, a.g)
         a.gen_key()
         b.gen_key()
