@@ -18,7 +18,7 @@ Others:
 """
 
 import os, socket, string, sys, tempfile, thread, time, unittest
-from M2Crypto import Rand, SSL, m2
+from M2Crypto import Rand, SSL, m2, Err
 
 srv_host = 'localhost'
 srv_port = 64000
@@ -49,7 +49,14 @@ class SSLClientTestCase(unittest.TestCase):
     def start_server(self, args):
         pid = os.fork()
         if pid == 0:
-            os.execvp('openssl', args)
+            # openssl must be started in the tests directory for it
+            # to find the .pem files
+            os.chdir('tests')
+            try:
+                os.execvp('openssl', args)
+            finally:
+                os.chdir('..')
+                
         else:
             time.sleep(0.5)
             return pid
@@ -97,6 +104,32 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+    def test_server_simple_secure_context(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/ca.pem')
+            s = SSL.Connection(ctx)
+            s.connect(self.srv_addr)
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_server_simple_secure_context_fail(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/server.pem')
+            s = SSL.Connection(ctx)
+            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            s.close()
+        finally:
+            self.stop_server(pid)
+
     def test_tls1_nok(self):
         self.args.append('-no_tls1')
         pid = self.start_server(self.args)
@@ -425,7 +458,7 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('ca.pem')
+            ctx.load_verify_locations('tests/ca.pem')
             s = SSL.Connection(ctx)
             try:
                 s.connect(self.srv_addr)
@@ -442,7 +475,7 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('server.pem')
+            ctx.load_verify_locations('tests/server.pem')
             s = SSL.Connection(ctx)
             self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
             s.close()
@@ -455,8 +488,8 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('ca.pem')
-            ctx.load_cert('x509.pem')
+            ctx.load_verify_locations('tests/ca.pem')
+            ctx.load_cert('tests/x509.pem')
             s = SSL.Connection(ctx)
             try:
                 s.connect(self.srv_addr)
@@ -474,8 +507,8 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('ca.pem')
-            ctx.load_cert('x509.pem')
+            ctx.load_verify_locations('tests/ca.pem')
+            ctx.load_cert('tests/x509.pem')
             s = SSL.Connection(ctx)
             try:
                 s.connect(self.srv_addr)
@@ -493,7 +526,7 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('ca.pem')
+            ctx.load_verify_locations('tests/ca.pem')
             s = SSL.Connection(ctx)
             self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
             s.close()
@@ -506,7 +539,7 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context()
             ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('ca.pem')
+            ctx.load_verify_locations('tests/ca.pem')
             s = SSL.Connection(ctx)
             self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
             s.close()
@@ -525,6 +558,34 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+    def test_HTTPSConnection_secure_context(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/ca.pem')
+            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
+            c.request('GET', '/')
+            data = c.getresponse().read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPSConnection_secure_context_fail(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/server.pem')
+            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
+            self.assertRaises(SSL.SSLError, c.request, 'GET', '/')
+            c.close()
+        finally:
+            self.stop_server(pid)
+
     def test_HTTPS(self):
         pid = self.start_server(self.args)
         try:
@@ -543,6 +604,43 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+    def test_HTTPS_secure_context(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/ca.pem')
+            c = httpslib.HTTPS(srv_host, srv_port, ssl_context=ctx)
+            c.putrequest('GET', '/')
+            c.putheader('Accept', 'text/html')
+            c.putheader('Accept', 'text/plain')
+            c.endheaders()
+            err, msg, headers = c.getreply()
+            assert err == 200, err
+            f = c.getfile()
+            data = f.read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPS_secure_context_fail(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/server.pem')
+            c = httpslib.HTTPS(srv_host, srv_port, ssl_context=ctx)
+            c.putrequest('GET', '/')
+            c.putheader('Accept', 'text/html')
+            c.putheader('Accept', 'text/plain')
+            self.assertRaises(SSL.SSLError, c.endheaders)
+            c.close()
+        finally:
+            self.stop_server(pid)
+
     def test_urllib(self):
         pid = self.start_server(self.args)
         try:
@@ -556,6 +654,59 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+    # XXX Don't actually know how to use m2urllib safely!
+    #def test_urllib_secure_context(self):
+    #def test_urllib_secure_context_fail(self):
+
+    # XXX Don't actually know how to use m2urllib safely!
+    #def test_urllib_safe_context(self):
+    #def test_urllib_safe_context_fail(self):
+
+    if sys.version_info >= (2,4):
+        def test_urllib2(self):
+            pid = self.start_server(self.args)
+            try:
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener()
+                opener.addheaders = [('Connection', 'close')]
+                u = opener.open('https://%s:%s/' % (srv_host, srv_port))
+                data = u.read()
+                u.close()
+            finally:
+                self.stop_server(pid)
+            self.failIf(string.find(data, 's_server -quiet -www') == -1)
+    
+        def test_urllib2_secure_context(self):
+            pid = self.start_server(self.args)
+            try:
+                ctx = SSL.Context()
+                ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+                ctx.load_verify_locations('tests/ca.pem')
+                
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener(ctx)
+                opener.addheaders = [('Connection', 'close')]           
+                u = opener.open('https://%s:%s/' % (srv_host, srv_port))
+                data = u.read()
+                u.close()
+            finally:
+                self.stop_server(pid)
+            self.failIf(string.find(data, 's_server -quiet -www') == -1)
+    
+        def test_urllib2_secure_context_fail(self):
+            pid = self.start_server(self.args)
+            try:
+                ctx = SSL.Context()
+                ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+                ctx.load_verify_locations('tests/server.pem')
+                
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener(ctx)
+                opener.addheaders = [('Connection', 'close')]
+                self.assertRaises(SSL.SSLError, opener.open, 'https://%s:%s/' % (srv_host, srv_port))
+            finally:
+                self.stop_server(pid)
+
     def test_blocking0(self):
         pid = self.start_server(self.args)
         try:
@@ -583,13 +734,65 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+    def test_makefile(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            bio = s.makefile('rw')
+            #s.close()  # XXX bug 6628?
+            bio.write('GET / HTTP/1.0\n\n')
+            bio.flush()
+            data = bio.read()
+            bio.close()
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_makefile_err(self):
+        def http_get(s):
+            s.send('GET / HTTP/1.0\n\n') 
+            resp = ''
+            while 1:
+                try:
+                    r = s.recv(4096)
+                    if not r:
+                        break
+                except SSL.SSLError, e: # s_server throws an 'unexpected eof'...
+                    #print e
+                    break
+                resp = resp + r 
+            return resp
+
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            f = s.makefile()
+            data = http_get(s)
+            s.close()
+            del f
+            f = None
+            del s
+            s = None
+            err_code = Err.peek_error_code()
+            assert not err_code, 'Unexpected error: %s' % err_code
+            err = Err.get_error()
+            assert not err, 'Unexpected error: %s' % err
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
     def test_twisted_wrapper(self):
-        #
-        # LEAK ALERT!
-        # If this method is commented out, the leak detection code does
-        # not find any leaks.
-        #
-        
         # Test only when twisted and ZopeInterfaces are present
         try:
             from twisted.internet.protocol import ClientFactory
@@ -597,6 +800,8 @@ class SSLClientTestCase(unittest.TestCase):
             from twisted.internet import reactor
             import M2Crypto.SSL.TwistedProtocolWrapper as wrapper
         except ImportError:
+            import warnings
+            warnings.warn('Skipping twisted wrapper test because twisted not found')
             return
         
         class EchoClient(LineReceiver):
@@ -644,8 +849,8 @@ class CheckerTestCase(unittest.TestCase):
         from M2Crypto import X509
 
         check = Checker.Checker(host=srv_host,
-                                peerCertHash='9594D272A975F58F4430511D15B4B7FF3D778113')
-        x509 = X509.load_cert('server.pem')
+                                peerCertHash='02C0CAD90DE6837700173A839BB8E84BF1F5B820')
+        x509 = X509.load_cert('tests/server.pem')
         assert check(x509, srv_host)
         self.assertRaises(Checker.WrongHost, check, x509, 'example.com')
         
@@ -656,7 +861,7 @@ class CheckerTestCase(unittest.TestCase):
 class ContextTestCase(unittest.TestCase):
     def test_ctx_load_verify_locations(self):
         ctx = SSL.Context()
-        self.assertRaises(AssertionError, ctx.load_verify_locations, None, None)
+        self.assertRaises(ValueError, ctx.load_verify_locations, None, None)
 
 def suite():
     suite = unittest.TestSuite()
@@ -693,9 +898,9 @@ if __name__ == '__main__':
         gc.set_debug(gc.DEBUG_LEAK & ~gc.DEBUG_SAVEALL)
     
     try:
-        Rand.load_file('../randpool.dat', -1) 
+        Rand.load_file('randpool.dat', -1) 
         unittest.TextTestRunner().run(suite())
-        Rand.save_file('../randpool.dat')
+        Rand.save_file('randpool.dat')
     finally:
         zap_servers()
 
